module SkirentalFrontendApi
    class Booking
        include NoBrainer::Document
        include NoBrainer::Document::Timestamps

        before_save :sync_with_backend

        field  :session_id,      		:type => String,                   :unique => true, :required => true,   :index => true
        field  :shop_id,         		:type => String,                                    :required => true,   :index => true
        field  :pre_payment_context,   	:type => Hash,     :default => {}
        field  :post_payment_context,  	:type => Hash,     :default => {}
        field  :booking_code, 			:type => String

        def transaction
            shop_settings = SkirentalApi::Client.settings(shop_id, "de")
            fail if pre_payment_context.nil?

            items = []
            duration = (Date.parse(pre_payment_context[:endDate]) - Date.parse(pre_payment_context[:startDate])).to_i
            currency = shop_settings[:money][:payment][:cur]
            total = 0
            pre_payment_context[:customers].each do |customer| # TODO: Schleifen-Logik umbauen… quantities der producs summieren, product_ids können als array an die API übergeben werden…
                customer[:products].each do |product|
                    # We expect that prices are independent to the lang. We store item id's therefore we do not access prices of packages.
                    price = SkirentalApi::Client.get_price_of_item(product[:id], Date.parse(pre_payment_context[:startDate]).strftime("%Y-%m-%d"), duration, customer[:customerGroup][:ageFrom], shop_id, "de", false)

                    fail if price.nil?

                    next if price[:netPrice].blank? || price[:netPrice] <= 0

                    item = {
                        :name => product[:name],
                        :sku => product[:id],
                        :price => price[:netPrice],
                        :currency => currency,
                        :quantity => 1 # A user can only select or unselect an item, there is no quantity field…
                    }
                    items.push(item)

                    total += price[:netPrice].to_d ## Use BigDecimal in order to avoid rounding issues.

                    product[:price] = price
                end
            end

            self.save

            {
                :item_list => {
                    :items => items
                },
                :amount =>  {
                    :total =>  total,
                    :currency =>  currency
                },
                :description =>  "Wir bedanken uns fuer Ihre Buchung!" # TODO: do we need a description???
            }
        end

        def sync_with_backend
            self.booking_code = SkirentalApi::Client.sync_booking_state(self)
        end
    end
end
